from nitrado.client import Client
from nitrado.game_server import GameServer


def assert_success(response):
    if not response:
        return
    if 'status' not in response or response['status'] != 'success':
        raise AssertionError(f"API returned: {response}")


class Service:
    CLIENT = Client.CLIENT

    @staticmethod
    def find_by_id(service_id):
        path = ['services', service_id]
        resp = Service.CLIENT.get(path=path)
        assert_success(resp)
        data = resp['data']['services']
        return Service(data)

    @staticmethod
    def all():
        services = []
        resp_servers = Service.CLIENT.get(path='services')
        assert_success(resp_servers)
        servers = resp_servers['data']['services']
        for data in servers:
            services.append(Service(data))
        return services

    def __init__(self, data):
        assert type(data) == dict, f"constructor only accepts type dict: Service({data})"
        self.id = data['id'] if 'id' in data else None
        self.location_id = data['location_id'] if 'location_id' in data else None
        self.status = data['status'] if 'status' in data else None
        self.status_code = data['status_code'] if 'status_code' in data else None
        self.websocket_token = data['websocket_token'] if 'websocket_token' in data else None
        self.user_id = data['user_id'] if 'user_id' in data else None
        self.username = data['username'] if 'username' in data else None
        self.comment = data['comment'] if 'comment' in data else None
        self.auto_extension = data['auto_extension'] if 'auto_extension' in data else None
        self.auto_extension_duration = data['auto_extension_duration'] if 'auto_extension_duration' in data else None
        self.auto_extension_external = data['auto_extension_external'] if 'auto_extension_external' in data else None
        self.type = data['type'] if 'type' in data else None
        self.type_human = data['type_human'] if 'type_human' in data else None
        self.details = data['details'] if 'details' in data else None
        self.start_date = data['start_date'] if 'start_date' in data else None
        self.suspend_date = data['suspend_date'] if 'suspend_date' in data else None
        self.delete_date = data['delete_date'] if 'delete_date' in data else None
        self.suspending_in = data['suspending_in'] if 'suspending_in' in data else None
        self.deleting_in = data['deleting_in'] if 'deleting_in' in data else None

    def game_server(self):
        return GameServer.find_by_id(self.id) if self.id else None

    def logs(self, page=None):
        if page:
            return Service.CLIENT.get(path=['services', self.id, 'logs'], data={'page': page})
        logs = []
        log = Service.CLIENT.get(path=['services', self.id, 'logs'], data={'page': page})
        logs += log['data']['logs']
        current_page = 2
        last_page = log['data']['page_count']
        while current_page < last_page:
            log = Service.CLIENT.get(path=['services', self.id, 'logs'], data={'page': page})
            logs += log['data']['logs']
            current_page += 1
        return logs

    def notifications(self):
        notify = Service.CLIENT.get(path=['services', self.id, 'notifications'])
        assert_success(notify)
        return notify['data']['notifications']

    def tasks(self):
        path = ['services', self.id, 'tasks']
        tasks = Service.CLIENT.get(path=path)
        assert_success(tasks)
        return tasks['data']['tasks']

    def create_task(self, action_method=None, month="*", day="*", hour="24", minute="0", weekday="*", action_data=None):
        path = ['services', self.id, 'tasks']
        params = {'action_method': action_method, 'action_data': action_data, 'minute': minute,
                  'hour': hour, 'day': day, 'month': month, 'weekday': weekday}
        try:
            Service.CLIENT.post(path=path, params=params)
            return True
        except Exception as e:
            print(e)
            return False

    def update_task(self, task_id=None, action_method=None, month="*", day="*", hour="*", minute="*", weekday="*", action_data=None):
        path = ['services', self.id, 'tasks', task_id]
        params = {'action_method': action_method, 'action_data': action_data, 'minute': minute,
                  'hour': hour, 'day': day, 'month': month, 'weekday': weekday}
        try:
            Service.CLIENT.put(path=path, params=params)
            return True
        except Exception as e:
            print(e)
            return False

    def delete_task(self, task_id):
        path = ['services', self.id, 'tasks', task_id]
        try:
            Service.CLIENT.delete(path=path)
            return True
        except Exception as e:
            print(e)
            return False

    def __repr__(self):
        return f"<Service(id={self.id}, username='{self.username}', details={self.details})>"

    def __str__(self):
        return f"""
        id = {self.id}
        location_id = {self.location_id}
        status = '{self.status}'
        status_code = {self.status_code}
        websocket_token = '{self.websocket_token}'
        user_id = {self.user_id}
        username = '{self.username}'
        comment = {self.comment}
        auto_extension = {self.auto_extension}
        auto_extension_duration = {self.auto_extension_duration}
        auto_extension_external = {self.auto_extension_external}
        type = '{self.type}'
        type_human = '{self.type_human}'
        details = {self.details}
        start_date = '{self.start_date}'
        suspend_date = '{self.suspend_date}'
        delete_date = '{self.delete_date}'
        suspending_in = {self.suspending_in}
        deleting_in = {self.deleting_in}
        """



from nitrado.client import Client
import requests
from pathlib import Path, WindowsPath


def assert_success(response):
    if not response:
        return
    if 'status' not in response or response['status'] != 'success':
        raise AssertionError(f"API returned: {response}")


def assert_request(req):
    if not req.ok:
        code = req.status_code
        reason = req.reason
        raise requests.RequestException(f"Url error => {code} {reason} for:", req.url)


class GameServer:
    CLIENT = Client.CLIENT

    @classmethod
    def find_by_id(cls, service_id):
        path = ['services', service_id, 'gameservers']
        resp = GameServer.CLIENT.get(path=path)
        assert_success(resp)
        data = resp['data']['gameserver']
        return GameServer(data)

    @classmethod
    def all(cls):
        games = []
        resp = GameServer.CLIENT.get(path='services')
        assert_success(resp)
        service_ids = [serv['id'] for serv in resp['data']['services']]
        for serv_id in service_ids:
            games.append(GameServer.find_by_id(serv_id))
        return games

    def __init__(self, data):
        """
        :param service_id: the gameserver id on nitrado
        """
        self.status = data['status'] if 'status' in data else None
        self.last_status_change = data['last_status_change'] if 'last_status_change' in data else None
        self.must_be_started = data['must_be_started'] if 'must_be_started' in data else None
        self.websocket_token = data['websocket_token'] if 'websocket_token' in data else None
        self.hostsystems = data['hostsystems'] if 'hostsystems' in data else None
        self.username = data['username'] if 'username' in data else None
        self.user_id = data['user_id'] if 'user_id' in data else None
        self.service_id = data['service_id'] if 'service_id' in data else None
        self.location_id = data['location_id'] if 'location_id' in data else None
        self.minecraft_mode = data['minecraft_mode'] if 'minecraft_mode' in data else None
        self.ip = data['ip'] if 'ip' in data else None
        self.ipv6 = data['ipv6'] if 'ipv6' in data else None
        self.port = data['port'] if 'port' in data else None
        self.query_port = data['query_port'] if 'query_port' in data else None
        self.rcon_port = data['rcon_port'] if 'rcon_port' in data else None
        self.label = data['label'] if 'label' in data else None
        self.type = data['type'] if 'type' in data else None
        self.memory = data['memory'] if 'memory' in data else None
        self.memory_mb = data['memory_mb'] if 'memory_mb' in data else None
        self.game = data['game'] if 'game' in data else None
        self.game_human = data['game_human'] if 'game_human' in data else None
        self.game_specific = data['game_specific'] if 'game_specific' in data else None
        self.modpacks = data['modpacks'] if 'modpacks' in data else None
        self.slots = data['slots'] if 'slots' in data else None
        self.location = data['location'] if 'location' in data else None
        self.credentials = data['credentials'] if 'credentials' in data else None
        self.settings = data['settings'] if 'settings' in data else None
        self.quota = data['quota'] if 'quota' in data else None
        self.query = data['query'] if 'query' in data else None

    def cluster_id(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'games', 'arkse', 'gen_cluster_id']
        resp = GameServer.CLIENT.get(path=path)
        assert_success(resp)
        return resp['data']['clusterid']

    def logs_shooter_game(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'download']
        params = {'file': f"/games/{self.username}/noftp/arkxb/ShooterGame/Saved/Logs/ShooterGame.log"}
        resp_url = GameServer.CLIENT.get(path=path, params=params)
        assert_success(resp_url)
        url = resp_url['data']['token']['url']
        req = requests.get(url)
        assert_request(req)
        return req.text.replace("\r\n", "\n")

    def logs_restart(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'download']
        params = {'file': f"/games/{self.username}/ftproot/restart.log"}
        resp_url = GameServer.CLIENT.get(path=path, params=params)
        assert_success(resp_url)
        url = resp_url['data']['token']['url']
        req = requests.get(url)
        assert_request(req)
        return req.text.replace("\r\n", "\n")

    def logs_shooter_game_last(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'download']
        params = {'file': f"/games/{self.username}/noftp/arkxb/ShooterGame/Saved/Logs/ShooterGame_Last.log"}
        resp_url = GameServer.CLIENT.get(path=path, params=params)
        assert_success(resp_url)
        url = resp_url['data']['token']['url']
        req = requests.get(url)
        assert_request(req)
        return req.text.replace("\r\n", "\n")

    def logs_download(self, directory=Path.cwd()):
        assert type(directory) == str or type(directory) == Path or type(
            directory) == WindowsPath, "Path provided must be of type string"
        location = Path(directory)
        if not location.is_dir():
            raise FileNotFoundError(f"The directory provided does not exist: {directory}")
        with open(location / Path('shooter_games.txt'), 'w') as w:
            w.write(self.logs_shooter_game())
        with open(location / Path('shooter_games_last.txt'), 'w') as w:
            w.write(self.logs_shooter_game_last())
        with open(location / Path('restart.txt'), 'w') as w:
            w.write(self.logs_restart())

    def start(self, game):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'games', 'start']
        params = {'game': game}
        return GameServer.CLIENT.post(path=path, params=params)

    def backup_restore(self, folder, backup_id):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'backups', 'gameserver']
        params = {'folder': folder, 'backup': backup_id}
        return GameServer.CLIENT.post(path=path, params=params)

    def download_file(self, file_path):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'download']
        params = {'path': file_path}
        return GameServer.CLIENT.get(path=path, params=params)

    def rename_file(self, file_path, target_path, target_name):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'move']
        params = {'source_path': file_path, 'target_path': target_path, 'target_filename': target_name}
        return GameServer.CLIENT.post(path=path, params=params)

    # def rename_file(self, file_path, new_name):
    #     path = ['services', self.service_id, 'gameservers', 'file_server', 'move']
    #     target_path = '/'.join(file_path.split("/")[:-1])
    #     params = {'source_path': file_path, 'target_path': target_path, 'target_filename': new_name}
    #     return self._client.post(path=path, params=params)

    def donation_history(self, page=0):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'boost', 'history']
        params = {'page': page}
        return GameServer.CLIENT.get(path=path, params=params)

    def donation_settings(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'boost']
        return GameServer.CLIENT.get(path=path)['data']

    def update_donation_settings(self, enable=True, message=None, welcome_message=None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'boost']
        params = {'enable': enable, 'message': message, 'welcome_message': welcome_message}
        return GameServer.CLIENT.put(path=path, params=params)

    def players(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ["services", self.service_id, "gameservers", "games", "players"]
        resp_players = GameServer.CLIENT.get(path=path)
        assert_success(resp_players)
        return resp_players['data']['players']

    def white_list_player(self, gamertag):
        """
        Player_Management - Add Player to Whitelist
        :param gamertag: Player unique identifier.
        """
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        params = {"identifer": gamertag}
        path = ["services", self.service_id, "gameservers", "games", "whitelist"]
        return GameServer.CLIENT.put(path=path, params=params)

    def make_admin(self, gamertag):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        """ Must whitelist the player first """
        path = ['services', self.service_id, 'gameservers', 'adminlist']
        params = {'identifier': gamertag}
        return GameServer.CLIENT.post(path=path, params=params)

    def details(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers']
        resp_details = GameServer.CLIENT.get(path=path)
        assert_success(resp_details)
        return resp_details['data']['gameserver']

    def restart(self, restart_message: str = None, log_message: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'restart']
        params = {"restart_message": restart_message, "message": log_message}
        resp_restart = GameServer.CLIENT.post(path=path, params=params)
        assert_success(resp_restart)
        return resp_restart['status'] == 'success'

    def stop(self, message=None, stop_message=None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        try:
            path = ['services', self.service_id, 'gameservers', 'stop']
            params = {'message': message, 'stop_message': stop_message}
            return GameServer.CLIENT.post(path=path, params=params)['status'] == 'success'
        except Exception as e:
            print(e)
            return False

    def list_backups(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'backups']
        return GameServer.CLIENT.get(path=path)

    def command(self, command):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'app_server', 'command']
        params = {'command': command}
        return GameServer.CLIENT.post(path=path, params=params)

    def ping(self, command):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'app_server']
        return GameServer.CLIENT.get(path=path, params={'command': command})

    def restore_database(self, database: str, timestamp: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'backups', 'database']
        params = {'database': database, 'timestamp': timestamp}
        return GameServer.CLIENT.post(path=path, params=params)

    def ftp_change_password(self, password: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'ftp', 'password']
        params = {'password': password}
        return GameServer.CLIENT.post(path=path, params=params)

    def bookmarks(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'bookmarks']
        return GameServer.CLIENT.get(path=path)

    def copy_file(self, source_path: str, target_path: str, target_name: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'copy']
        params = {'source_path': source_path, 'target_path': target_path, 'target_name': target_name}
        return GameServer.CLIENT.post(path=path, params=params)

    def create_directory(self, dir_path, name):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'mkdir']
        params = {'path': dir_path, 'name': name}
        return GameServer.CLIENT.post(path=path, params=params)

    def delete_file(self, file_path):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'delete']
        params = {'path': file_path}
        return GameServer.CLIENT.delete(path=path, params=params)

    def list_files(self, dir_path=None, search: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'list']
        params = {'dir': dir_path, 'search': search}
        return GameServer.CLIENT.get(path=path, params=params)

    def move_file(self, source_path: str, target_path: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'move']
        name = source_path.split('/')[-1]
        params = {'source_path': source_path, 'target_path': target_path, 'target_file_name': name}
        return GameServer.CLIENT.post(path=path, params=params)

    def seek_file(self, file_path: str, offset: int, length: int, mode: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'seek']
        params = {'file': file_path, 'offset': offset, 'length': length, 'mode': mode}
        return GameServer.CLIENT.get(path=path, params=params)

    def file_size(self, file_path):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'size']
        params = {'file': file_path}
        return GameServer.CLIENT.get(path=path, params=params)

    def duplicate_file(self, source_path, target_path, target_name):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'copy']
        params = {'source_path': source_path, 'target_path': target_path, 'target_name': target_name}
        return GameServer.CLIENT.post(path=path, params=params)

    def files_stat(self, files_paths: list):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'stat']
        params = {'files': files_paths}
        return GameServer.CLIENT.get(path=path, params=params)

    def upload_file(self, target_dir: str, file_name: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'file_server', 'upload']
        params = {'path': target_dir, 'file': file_name}
        return GameServer.CLIENT.post(path=path, params=params)

    def list_all_games(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['gameserver', 'games']
        return GameServer.CLIENT.get(path=path)

    def install_game(self, game: str, modpack: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'games', 'install']
        params = {'game': game, 'modpack': modpack}
        return GameServer.CLIENT.post(path=path, params=params)

    def list_games(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'games']
        return GameServer.CLIENT.get(path=path)

    def start_game(self, game: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'games', 'start']
        params = {'game': game}
        return GameServer.CLIENT.post(path=path, params=params)

    def uninstall_game(self, game: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'games', 'uninstall']
        params = {'game': game}
        return GameServer.CLIENT.delete(path=path, params=params)

    def change_mysql_password(self, password: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'mysql', 'password']
        params = {'password': password}
        return GameServer.CLIENT.post(path=path, params=params)

    def reset_mysql_database(self, password: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'mysql', 'reset']
        params = {'password': password}
        return GameServer.CLIENT.post(path=path, params=params)

    def install_package(self, package: str, version: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'packages']
        params = {'package': package, 'version': version}
        return GameServer.CLIENT.post(path=path, params=params)

    def list_packages(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'packages']
        return GameServer.CLIENT.post(path=path)

    def reinstall_package(self, package: str, version: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'packages', 'reinstall']
        params = {'package': package, 'version': version}
        return GameServer.CLIENT.put(path=path, params=params)

    def uninstall_package(self, package: str):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'packages', 'uninstall']
        params = {'package': package}
        return GameServer.CLIENT.delete(path=path, params=params)

    def resource_usage(self, hours: int = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'stats']
        params = {'hours': hours}
        return GameServer.CLIENT.get(path=path, params=params)

    def create_settings_sets(self, name: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings', 'sets']
        params = {'name': name}
        return GameServer.CLIENT.post(path=path, params=params)

    def delete_settings_sets(self, set_id: int):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings', 'sets', set_id]
        return GameServer.CLIENT.delete(path=path)

    def settings_sets(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings', 'sets']
        sets = GameServer.CLIENT.get(path=path)['data']['sets']
        return [data['data'] for data in sets]

    def restore_settings_sets(self, set_id: int):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings', 'sets', set_id, 'restore']
        return GameServer.CLIENT.post(path=path)

    def default_settings(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings', 'defaults']
        return GameServer.CLIENT.get(path=path)

    def reset_settings(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings']
        return GameServer.CLIENT.delete(path=path)

    def update_settings(self, category: str = None, key: str = None, value: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'settings']
        params = {'category': category, 'key.txt': key, 'value': value}
        return GameServer.CLIENT.post(path=path, params=params)

    def boost_history(self, page: int = 0):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'boost', 'history']
        params = {'page': page}
        return GameServer.CLIENT.get(path=path, params=params)

    def boost_settings(self):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'boost']
        return GameServer.CLIENT.get(path=path)

    def update_boost_settings(self, enable: bool = True, message: str = None, welcome_message: str = None):
        assert GameServer.CLIENT is not None, "A client must be initialized for GameServer"
        path = ['services', self.service_id, 'gameservers', 'boost']
        params = {'enable': enable, 'message': message, 'welcome_message': welcome_message}
        return GameServer.CLIENT.put(path=path, params=params)

    def __repr__(self):
        return f"<GameServer(service_id={self.service_id}, status='{self.status}', query={self.query})>"

    def __str__(self):
        return f"""
        status = '{self.status}'
        last_status_change = {self.last_status_change}
        must_be_started = {self.must_be_started}
        websocket_token = '{self.websocket_token}'
        hostsystems = {self.hostsystems}
        username = '{self.username}'
        user_id = {self.user_id}
        service_id = {self.service_id}
        location_id = {self.location_id}
        minecraft_mode = {self.minecraft_mode}
        ip = '{self.ip}'
        ipv6 = {self.ipv6}
        port = {self.port}
        query_port = {self.query_port}
        rcon_port = {self.rcon_port}
        label = '{self.label}'
        type = '{self.type}'
        memory = '{self.memory}'
        memory_mb = {self.memory_mb}
        game = '{self.game}'
        game_human = {self.game_human}
        game_specific = {self.game_specific}
        modpacks = {self.modpacks}
        slots = {self.slots}
        location = '{self.location}'
        credentials = {self.credentials}
        settings = {self.settings}
        quota = {self.quota}
        query = {self.query}
        """



from nitrado.client import Client
from nitrado.game_server import GameServer
from nitrado.service import Service
import requests
from requests import RequestException
import os


def assert_success(response):
    if not response:
        return
    if 'status' not in response or response['status'] != 'success':
        raise AssertionError(f"API returned: {response}")


class NitradoAPI:
    NITRADO_API_URL = "https://api.nitrado.net/"
    CLIENT = Client.CLIENT

    @classmethod
    def initialize_client(cls, key=None, url=None):
        if not (Client.CLIENT or GameServer.CLIENT or Service.CLIENT or NitradoAPI.CLIENT):
            key = key or os.getenv("NITRADO_KEY")
            url = url or NitradoAPI.NITRADO_API_URL
            assert key and url, f"The url and api key must be provided: url=>{url}, key=>{key}"
            client = Client(url, key)
            Client.CLIENT = client
            GameServer.CLIENT = client
            Service.CLIENT = client
            NitradoAPI.CLIENT = client

    @classmethod
    def unofficial_server_list(cls):
        req = requests.get("http://arkdedicated.com/xbox/cache/unofficialserverlist.json")
        if not req.ok:
            code = req.status_code
            reason = req.reason
            raise RequestException(f"Url error => {code} {reason} for:", req.url)
        return req.json()

    @classmethod
    def official_server_list(cls):
        req = requests.get("http://arkdedicated.com/xbox/cache/officialserverlist.json")
        if not req.ok:
            code = req.status_code
            reason = req.reason
            raise RequestException(f"Url error => {code} {reason} for:", req.url)
        return req.json()

    @classmethod
    def banned_list(cls):
        req = requests.get("http://arkdedicated.com/xboxbanlist.txt")
        return req.text.split('\r\n')

    def __init__(self, key=None, url=None):
        NitradoAPI.initialize_client(key, url)
        self.services = Service.all()
        self.game_servers = GameServer.all()

    def health_check(self):
        resp = NitradoAPI.CLIENT.get('ping')
        assert_success(resp)
        return resp['message']

    def maintenance_status(self):
        resp = NitradoAPI.CLIENT.get('maintenance')
        assert_success(resp)
        return resp['data']['maintenance']

    def version(self):
        resp = NitradoAPI.CLIENT.get('version')
        assert_success(resp)
        return resp['message']



from nitrado.nitrado_api import NitradoAPI
from nitrado.service import Service
from nitrado.game_server import GameServer
from nitrado.client import Client


__all__ = ['NitradoAPI', 'Service', 'GameServer', 'Client', 'initialize_client']


def initialize_client(key=None, url=None):
    NitradoAPI.initialize_client(key, url)



from requests import get, post, put, delete, RequestException
import json


def pretty_json(data: dict):
    return json.dumps(data, indent=3, sort_keys=True)


def assert_response(response):
    if not response.ok:
        code = response.status_code
        reason = response.reason
        raise RequestException(f"Url error => {code} {reason} for:", response.url)


def get_content(response):
    content_type = response.headers['content-type'].split("; ")
    if 'application/json' in content_type:
        return response.json()
    return response.text


class Client:
    CLIENT = None

    def __init__(self, api_url, key=None):
        assert type(api_url) == str, "A string url must be provided in Client(api_url, key)"
        assert len(api_url) > 1, "Api URL's should include 'http://' or 'https://'"
        self.__headers = {'Authorization': key}
        self.__api_url = api_url if api_url[-1] == '/' else f"{api_url}/"
        Client.CLIENT = self

    def __make_path(self, path=None):
        if isinstance(path, str):
            return "{}{}".format(self.__api_url, path)
        elif isinstance(path, list):
            path_list = [str(directory) for directory in path]
            return "{}{}".format(self.__api_url, '/'.join(path_list))
        else:
            return self.__api_url

    def get(self, path=None, data=None, params=None):
        response = get(self.__make_path(path), headers=self.__headers, data=data, params=params)
        assert_response(response)
        return get_content(response)

    def post(self, path=None, data=None, params=None):
        response = post(self.__make_path(path), headers=self.__headers, data=data, params=params)
        assert_response(response)
        return get_content(response)

    def delete(self, path=None, data=None, params=None):
        response = delete(self.__make_path(path), headers=self.__headers, data=data, params=params)
        assert_response(response)
        return get_content(response)

    def put(self, path=None, data=None, params=None):
        response = put(self.__make_path(path), headers=self.__headers, data=data, params=params)
        assert_response(response)
        return get_content(response)
